@@ -1,5 +1,5 @@
 # pyimpspec is licensed under the GPLv3 or later (https://www.gnu.org/licenses/gpl-3.0.html).
-# Copyright 2022 pyimpspec developers
+# Copyright 2023 pyimpspec developers
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,18 +20,14 @@
 from os.path import exists
 from pyimpspec.data.data_set import (
     DataSet,
-    dataframe_to_dataset,
-)
-from pandas import (
-    DataFrame,
-    read_excel,
+    dataframe_to_data_sets,
 )
 from typing import List
 
 
 def parse_spreadsheet(path: str, **kwargs) -> List[DataSet]:
     """
-    Parse a spreadsheet (.xlsx or .ods) containing one ore more impedance spectra.
+    Parse a spreadsheet (.xlsx or .ods) containing one or more impedance spectra.
 
     Parameters
     ----------
@@ -39,20 +35,24 @@ def parse_spreadsheet(path: str, **kwargs) -> List[DataSet]:
         The path to the file to process.
 
     kwargs:
-        Keyword arguments (e.g. sheet names) to pass on to the parser. See the pandas.read_excel documentation for a list of the supported keyword arguments.
+        Keyword arguments are passed forward to `pandas.read_excel`_.
 
     Returns
     -------
     List[DataSet]
     """
-    assert type(path) is str and exists(path), path
+    from pandas import (
+        DataFrame,
+        read_excel,
+    )
+    assert isinstance(path, str) and exists(path), path
     data_sets: List[DataSet] = []
     if "sheet_name" not in kwargs:
         kwargs["sheet_name"] = None
     label: str
     df: DataFrame
     for label, df in read_excel(path, **kwargs).items():
-        data_sets.append(dataframe_to_dataset(df, path, label))
-    assert type(data_sets) is list, data_sets
+        data_sets.extend(dataframe_to_data_sets(df, path=path, label=label))
+    assert isinstance(data_sets, list), data_sets
     assert all(map(lambda _: isinstance(_, DataSet), data_sets))
     return data_sets
